// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "TFlowBars.h"
#include "TFlowBarsDlg.h"

STDMETHODIMP CTFlowBarsDlg::OnDataError(
                           LPDISPATCH /*obj*/,
                           BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CTFlowBarsDlg::OnDataConnectionStatusChanged(
                           eConnectionStatus newStatus)
{
   try
   {
      m_lblDataConnection.SetWindowText(_T("Data ") + CHelper::ConnectionStatusToString(newStatus));

      EnableControls();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTFlowBarsDlg::OnTFlowBarsResolved(
                           ICQGTFlowBars* cqg_trade_flow_bars,
                           ICQGError* /*cqg_error*/)
{
   try
   {
      if (!m_spTFlowBars)
      {
         return S_OK;
      }

      // Get selected tflow bars ID
      ATL::CComBSTR selTFlowBarsID;
      HRESULT hr = m_spTFlowBars->get_Id(&selTFlowBarsID);
      Common::ThrowOnCOMError(m_spTFlowBars, hr);

      // Get changed tflow bars ID
      ATL::CComBSTR tFlowBarsID;
      hr = cqg_trade_flow_bars->get_Id(&tFlowBarsID);
      Common::ThrowOnCOMError<ICQGTFlowBars>(cqg_trade_flow_bars, hr);

      // Check if tflow bars resolved in the selected request
      if (selTFlowBarsID == tFlowBarsID)
      {
         ClearAllData();
         ShowAllData();

         EnableControls();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTFlowBarsDlg::OnTFlowBarsAdded(
                           ICQGTFlowBars* cqg_trade_flow_bars)
{
   try
   {
      if (!m_spTFlowBars)
      {
         return S_OK;
      }

      // Get selected tflow bars ID
      ATL::CComBSTR selTFlowBarsID;
      HRESULT hr = m_spTFlowBars->get_Id(&selTFlowBarsID);
      Common::ThrowOnCOMError(m_spTFlowBars, hr);

      // Get changed tflow bars ID
      ATL::CComBSTR tFlowBarsID;
      hr = cqg_trade_flow_bars->get_Id(&tFlowBarsID);
      Common::ThrowOnCOMError<ICQGTFlowBars>(cqg_trade_flow_bars, hr);

      // Check if record added in the selected request
      if (selTFlowBarsID == tFlowBarsID)
      {
         // Show changed records
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTFlowBarsDlg::OnTFlowBarsInserted(
                           ICQGTFlowBars* cqg_trade_flow_bars,
                           long /*index_*/)
{
   try
   {
      if (!m_spTFlowBars)
      {
         return S_OK;
      }

      // Get selected tflow bars ID
      ATL::CComBSTR selTFlowBarsID;
      HRESULT hr = m_spTFlowBars->get_Id(&selTFlowBarsID);
      Common::ThrowOnCOMError(m_spTFlowBars, hr);

      // Get changed tflow bars ID
      ATL::CComBSTR tFlowBarsID;
      hr = cqg_trade_flow_bars->get_Id(&tFlowBarsID);
      Common::ThrowOnCOMError<ICQGTFlowBars>(cqg_trade_flow_bars, hr);

      // Check if record inserted to the selected request
      if (selTFlowBarsID == tFlowBarsID)
      {
         // Show changed records
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTFlowBarsDlg::OnTFlowBarsRemoved(
                           ICQGTFlowBars* cqg_trade_flow_bars,
                           long /*index_*/)
{
   try
   {
      if (!m_spTFlowBars)
      {
         return S_OK;
      }

      // Get selected tflow bars ID
      ATL::CComBSTR selTFlowBarsID;
      HRESULT hr = m_spTFlowBars->get_Id(&selTFlowBarsID);
      Common::ThrowOnCOMError(m_spTFlowBars, hr);

      // Get changed tflow bars ID
      ATL::CComBSTR tFlowBarsID;
      hr = cqg_trade_flow_bars->get_Id(&tFlowBarsID);
      Common::ThrowOnCOMError<ICQGTFlowBars>(cqg_trade_flow_bars, hr);

      // Check if record removed from the selected request
      if (selTFlowBarsID == tFlowBarsID)
      {
         // Show changed records
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTFlowBarsDlg::OnTFlowBarsUpdated(
                           ICQGTFlowBars* cqg_trade_flow_bars,
                           long index_)
{
   try
   {
      if (!m_spTFlowBars)
      {
         return S_OK;
      }

      // Get selected tflow bars ID
      ATL::CComBSTR selTFlowBarsID;
      HRESULT hr = m_spTFlowBars->get_Id(&selTFlowBarsID);
      Common::ThrowOnCOMError(m_spTFlowBars, hr);

      // Get changed tflow bars ID
      ATL::CComBSTR tFlowBarsID;
      hr = cqg_trade_flow_bars->get_Id(&tFlowBarsID);
      Common::ThrowOnCOMError<ICQGTFlowBars>(cqg_trade_flow_bars, hr);

      // Check if record updated in the selected request
      if (selTFlowBarsID == tFlowBarsID)
      {
         // Find row for specified tflow bar
         CString indexStr;
         indexStr.Format(_T("%d"), index_ + 1);
         LVFINDINFO findInfo;
         findInfo.flags = LVFI_STRING;
         findInfo.psz = indexStr;
         int row = m_lvLastTFlowBars.FindItem(&findInfo);

         // If found change it's properties
         if (row > -1)
         {
            ShowRecord(index_ + 1, m_lvLastTFlowBars, row);
         }
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}